
//
//  PatternModel.m
//
//  Created by Dan Grassi on 7/8/11.
//  Copyright 2011 Zaph, LLC. All rights reserved.
//

#import "PatternModel.h"
#import "Tower.h"
#import "Utility.h"
#import <math.h>

@interface PatternModel ()
@property (nonatomic) double fieldRMS;
@property (nonatomic) float fieldMax;
@end


@implementation PatternModel


- (id)init {
    self = [super init];
    if (self) {
        self.towers = [NSMutableArray array];
        _fieldMax   = 0.0;
        _fieldRMS   = 1.0;
    }
    
    return self;
}

- (void)addTower:(Tower *)tower {
    [self.towers addObject:tower];
}

- (void)removeTower:(Tower *)tower {
    [self.towers removeObject:tower];
}

- (Tower *)towerNumber:(NSUInteger)number {
    Tower *tower = nil;
    
    if (number < self.towers.count)
        tower = [self.towers objectAtIndex:number];
    
    return tower;
}

- (Tower *)lastTower {
    return self.towers.lastObject;
}

- (NSUInteger)towerCount {
    return self.towers.count;
}

- (NSUInteger)indexOfTower:(Tower *)tower {
    return [self.towers indexOfObject:tower];
}

- (NSMutableArray *)createPatternPoints {
    NSMutableArray *points = [NSMutableArray array];
    
    [self calculatePoints:points];
    [self normalizePoints:points];

	return points;
}

- (void)calculatePoints:(NSMutableArray *)points {
    int segmentCount = 120;
    float fullCircle = M_PI * 2;
    float increment  = fullCircle / segmentCount;
    self.fieldMax = 0;

    float nomalizer = [self fieldNomanizationFactor];
    for (int i=0; i<=segmentCount; i+=1) {
        float phi = increment * i;
        float field = [self fieldValueForAzimuth:phi nomalizer:nomalizer];

        if (self.fieldMax < field)
            self.fieldMax = field;

        FieldPoint *point = [FieldPoint pointWithMagnitude:field angle:phi];
        [points addObject:point];
    }
}

- (float)calculatedRMSWithPoints:(NSArray *)points {
    float rms = 0.0;
    for (FieldPoint *point in points) {
        rms += point.magnitude * point.magnitude;
    }
    rms /= points.count;
    rms = sqrt(rms);

    float totalField = 0.0;
    for (Tower *tower in self.towers) {
        totalField += tower.field;
    }
    rms /= totalField;

    return rms;
}

- (float)fieldValueForAzimuth:(float)azimuth nomalizer:(float)nomalizer {
    float fieldValue = 0;
    float vectorX = 0;
    float vectorY = 0;

    azimuth = [Utility normalizeAngle:azimuth];

    for (Tower *tower in self.towers) {
        float cosAzimuth = cosf(tower.orientation - azimuth);
        vectorX += tower.field * sin(tower.phase + (tower.spacing * cosAzimuth));
        vectorY += tower.field * cos(tower.phase + (tower.spacing * cosAzimuth));
    }

    vectorX *= nomalizer;
    vectorY *= nomalizer;

    fieldValue = hypotf(vectorX, vectorY);

    return fieldValue;
}

- (void)normalizePoints:(NSMutableArray *)points {
    float scale = 1.0 / self.fieldMax;
    if (isinf(scale) == NO) {
        [self scalePoints:points scale:1/self.fieldMax];
    }
}

- (void)scalePoints:(NSMutableArray *)points scale:(float)scale {
    for (FieldPoint *point in points) {
        point.magnitude *= scale;
    }
}

- (void)normalizeFields {
    if (self.towers.count <2) {
        return;
    }

    float factor = [self fieldNomanizationFactor];
    for (Tower *tower in self.towers) {
        tower.field *= factor;
    }
}

- (float)fieldNomanizationFactor {
    float totalField = 0.0;
    for (Tower *tower in self.towers) {
        totalField += tower.field;
    }

    if (totalField < 0.01) { // Simplistic fix
//        NSLog(@"Simplistic fix, totalField < 0.01");
        totalField = 0.01;
    }

    float factor = 1.0 / totalField;

    return factor;
}

- (void)calculateRMSField {
    self.fieldRMS = 0;
    float et      = 0;

    for (Tower *towerI in self.towers) {
        et += towerI.field;
        for (Tower *towerJ in self.towers) {
            self.fieldRMS += towerI.field
                           * towerJ.field
                           * cos(towerJ.phase   - towerI.phase)
                           * j0( towerJ.spacing - towerI.spacing);
        }
    }

    if (et >= 0)
        self.fieldRMS = sqrt(self.fieldRMS) / et;
}

- (NSString *)description {
    return [NSString stringWithFormat:@"PatternModel - fieldMax: %3.f, fieldRMS: %3.f, towers: %@", self.fieldMax, self.fieldRMS, self.towers];
}

@end
